@@ -25,7 +25,6 @@ def subnet_size(host_count):
     host_count += 2 # network and broadcast
     return int(math.ceil(math.log(host_count, 2)))
 
-
 @functools.total_ordering
 class TreeNode(object):
     def __init__(self, graph, node):
@@ -44,7 +43,6 @@ class TreeNode(object):
             return self.host < other.host
         return self.node < other.node
 
-
 #TODO: restore function that truncated subnets
 
     def __repr__(self):
@@ -425,10 +423,10 @@ class IpTree(object):
         for n in interfaces:
             n.host.loopback = n.subnet
 
-def assign_asn_to_interasn_cds(G_ip):
-    G_phy = G_ip.overlay("phy")
-    for collision_domain in G_ip.nodes("collision_domain"):
-        neigh_asn = list(ank_utils.neigh_attr(G_ip, collision_domain, "asn", G_phy)) #asn of neighbors
+def assign_asn_to_interasn_cds(g_ip):
+    G_phy = g_ip.overlay("phy")
+    for collision_domain in g_ip.nodes("collision_domain"):
+        neigh_asn = list(ank_utils.neigh_attr(g_ip, collision_domain, "asn", G_phy)) #asn of neighbors
         if len(set(neigh_asn)) == 1:
             asn = set(neigh_asn).pop() # asn of any neigh, as all same
         else:
@@ -437,23 +435,24 @@ def assign_asn_to_interasn_cds(G_ip):
 
     return
 
-def allocate_ips(G_ip, infrastructure = True):
+def allocate_ips(g_ip, infrastructure = True, loopbacks = True):
     """Can disable infrastructure, eg for ipv6, still want to alloc ipv4 loopbacks for router ids"""
-    log.info("Allocating Primary Host loopback IPs")
-    ip_tree = IpTree("192.168.1.0")
-    ip_tree.add_nodes(G_ip.nodes("is_l3device"))
-    ip_tree.build()
-    loopback_tree = ip_tree.json()
-   # json.dumps(ip_tree.json(), cls=autonetkit.ank_json.AnkEncoder, indent = 4)
-    #body = json.dumps({"ip_allocations": jsontree})
-    #messaging.publish_compressed("www", "client", body)
-    ip_tree.assign()
-    G_ip.data.loopback_blocks = ip_tree.group_allocations()
+    if loopbacks:
+        log.info("Allocating v4 Primary Host loopback IPs")
+        ip_tree = IpTree("192.168.1.0")
+        ip_tree.add_nodes(g_ip.nodes("is_l3device"))
+        ip_tree.build()
+        loopback_tree = ip_tree.json()
+    # json.dumps(ip_tree.json(), cls=autonetkit.ank_json.AnkEncoder, indent = 4)
+        #body = json.dumps({"ip_allocations": jsontree})
+        #messaging.publish_compressed("www", "client", body)
+        ip_tree.assign()
+        g_ip.data.loopback_blocks = ip_tree.group_allocations()
 
-    log.info("Allocating Secondary Host loopback IPs")
+    log.debug("Allocating v4 Secondary Host loopback IPs")
     ip_tree = IpTree("172.16.0.0")
     secondary_loopbacks = []
-    for n in G_ip.nodes():
+    for n in g_ip.nodes():
         for i in n.interfaces("is_loopback"):
             secondary_loopbacks.append(i)
 
@@ -464,14 +463,14 @@ def allocate_ips(G_ip, infrastructure = True):
     #body = json.dumps({"ip_allocations": jsontree})
     #messaging.publish_compressed("www", "client", body)
     ip_tree.assign()
-    #G_ip.data.loopback_blocks = ip_tree.group_allocations()
+    #g_ip.data.loopback_blocks = ip_tree.group_allocations()
 
-    log.info("Allocating Collision Domain IPs")
 
     if infrastructure:
+        log.info("Allocating v4 Infrastructure IPs")
         ip_tree = IpTree("10.0.0.0")
-        assign_asn_to_interasn_cds(G_ip)
-        ip_tree.add_nodes(G_ip.nodes("collision_domain"))
+        assign_asn_to_interasn_cds(g_ip)
+        ip_tree.add_nodes(g_ip.nodes("collision_domain"))
         ip_tree.build()
         cd_tree = ip_tree.json()
         ip_tree.assign()
@@ -492,6 +491,6 @@ def allocate_ips(G_ip, infrastructure = True):
     messaging.publish_compressed("www", "client", body)
 
 #TODO: need to update with loopbacks if wish to advertise also - or subdivide blocks?
-    G_ip.data.infra_blocks = ip_tree.group_allocations()
+    g_ip.data.infra_blocks = ip_tree.group_allocations()
 
     #ip_tree.save()
